@@ -3,6 +3,7 @@
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
+ *	Juan-Mariano de Goyeneche <jmseyas@dit.upm.es> (BLOB issues)
  *
  * This Library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public License as
@@ -27,6 +28,7 @@
 #include <glib/gfileutils.h>
 #include <glib/gmessages.h>
 #include <glib/gstrfuncs.h>
+#include <libsql/sql_parser.h>
 #include <libgda/gda-intl.h>
 #include <libgda/gda-log.h>
 #include <libgda/gda-util.h>
@@ -45,6 +47,7 @@ gda_type_to_string (GdaValueType type)
 	switch (type) {
 	case GDA_VALUE_TYPE_NULL : return "null";
 	case GDA_VALUE_TYPE_BIGINT : return "bigint";
+	case GDA_VALUE_TYPE_BLOB : return "blob";
 	case GDA_VALUE_TYPE_BINARY : return "binary";
 	case GDA_VALUE_TYPE_BOOLEAN : return "boolean";
 	case GDA_VALUE_TYPE_DATE : return "date";
@@ -76,6 +79,7 @@ gda_type_from_string (const gchar *str)
 	if (!g_strcasecmp (str, "null")) return GDA_VALUE_TYPE_NULL;
 	else if (!g_strcasecmp (str, "bigint")) return GDA_VALUE_TYPE_BIGINT;
 	else if (!g_strcasecmp (str, "binary")) return GDA_VALUE_TYPE_BINARY;
+	else if (!g_strcasecmp (str, "blob")) return GDA_VALUE_TYPE_BLOB;
 	else if (!g_strcasecmp (str, "boolean")) return GDA_VALUE_TYPE_BOOLEAN;
 	else if (!g_strcasecmp (str, "date")) return GDA_VALUE_TYPE_DATE;
 	else if (!g_strcasecmp (str, "double")) return GDA_VALUE_TYPE_DOUBLE;
@@ -116,6 +120,39 @@ gda_string_hash_to_list (GHashTable *hash_table)
         return list;
 }
 
+/**
+ * gda_sql_replace_placeholders
+ * @sql: A SQL command containing placeholders for values.
+ * @params: List of values for the placeholders.
+ *
+ * Replaces the placeholders (:name) in the given SQL command with
+ * the values from the #GdaParameterList specified as the @params
+ * argument.
+ *
+ * Returns: the SQL string with all placeholders replaced, or NULL
+ * on error. On success, the returned string must be freed by the caller
+ * when no longer needed.
+ */
+gchar *
+gda_sql_replace_placeholders (const gchar *sql, GdaParameterList *params)
+{
+	GString *str;
+	sql_statement *sql_stmt;
+
+	g_return_val_if_fail (sql != NULL, NULL);
+
+	/* parse the string */
+	sql_stmt = sql_parse (sql);
+	if (!sql_stmt) {
+		gda_log_error (_("Could not parse SQL command '%s'"), sql);
+		return NULL;
+	}
+
+	/* FIXME */
+
+	return NULL;
+}
+
 /**
  * gda_file_load
  * @filename: path for the file to be loaded.
@@ -164,7 +201,7 @@ gda_file_save (const gchar *filename, const gchar *buffer, gint len)
 	
 	g_return_val_if_fail (filename != NULL, FALSE);
 
-	fd = open (filename, O_RDWR | O_CREAT);
+	fd = open (filename, O_RDWR | O_CREAT, 0644);
 	if (fd == -1) {
 		gda_log_error (_("Could not create file %s"), filename);
 		return FALSE;
